package engcalculator.variable;

import engcalculator.variable.physicalConstant.*;
import engcalculator.variable.convertConstant.*;
import engcalculator.data.Interval;
import engcalculator.data.IntervalList;
import engcalculator.data.NamedMatrixIntervalList;
import engcalculator.function.CalculusException;
import engcalculator.function.prefix.PrefixFunction;
import engcalculator.expression.token.PrefixFunctionTkn;
import java.util.Collection;
import java.util.HashMap;

/*
 * This interface is used by the Expression
 * class so that it can solve for variables.
 */

public class Variable extends PrefixFunction {
    private static HashMap<String, Variable> VARIABLES = new HashMap <String, Variable> ();
    private final static PrefixFunctionTkn PREFIX_FUNCTION_TKN = new PrefixFunctionTkn();

    public static boolean addVariable (Variable var) throws Exception {
        if(! PREFIX_FUNCTION_TKN.mayBeCompatible(var.getSymbol())) throw new CalculusException("The variable name is not properly formed: "+var.getSymbol());
        
        if (var.getNumbArgs() != 0) throw new RuntimeException("It was defined a variable with arguments!?");

        Variable v;
        if ((v = VARIABLES.get(var.getSymbol())) != null && v.isConstant()) return false;

        VARIABLES.put(var.getSymbol(), var);

        return true;
    }

    public static boolean delVariable (String symbol) {
        Variable v;
        if ((v = VARIABLES.get(symbol)) != null && ! v.isConstant()) {
            VARIABLES.remove(symbol);
            return true;
        }
        else return false;
    }

    public static Variable getVariable (String symbol) {
        return VARIABLES.get(symbol);
    }

    public static Collection<Variable> getVariables () {
        return VARIABLES.values();
    }

    public static void initialize () throws Exception {
//        addVariable(new Empty());
        addVariable(new E());
        addVariable(new PI());

        addVariable(new AtomicMass());
        addVariable(new Avogadro());
        addVariable(new Boltzmann());
        addVariable(new ConductanceQuantum());
        addVariable(new Electric());
        addVariable(new ElectronMass());
        addVariable(new ElectronVolt());
        addVariable(new ElementaryCharge());
        addVariable(new Faraday());
        addVariable(new FineStructure());
        addVariable(new Magnetic());
        addVariable(new MagneticFluxQuantum());
        addVariable(new MolarGasConstant());
        addVariable(new Newton());
        addVariable(new Planck());
        addVariable(new ProtonMass());
        addVariable(new Rydberg());
        addVariable(new SpeedOfLight());
        addVariable(new StefanBoltzmann());

        addVariable(new InchesToMeter());
        addVariable(new FootToMeter());
        addVariable(new YardToMeter());

        addVariable(new PoundToKilogram());
        addVariable(new OunceToKilogram());

        addVariable(new YearToSecond());
        addVariable(new MonthToSecond());
        addVariable(new DayToSecond());
        addVariable(new HourToSecond());

        addVariable(new Knot2MeterPerSecond());
        addVariable(new MilePerHour2MeterPerSecond());
        addVariable(new KilometerPerHour2MeterPerSecond());
        addVariable(new FootPerHour2MeterPerSecond());

        addVariable(new DegreeToRadiant());
    }

    private NamedMatrixIntervalList value;

    public Variable(String name, IntervalList value, boolean constant, String help) {
        super (name, (byte) 0, false, constant, help, null, null);
        this.value = new NamedMatrixIntervalList (value, name);
    }

    public Variable(String name, Interval value, boolean constant, String help) {
        super (name, (byte) 0, false, constant, help, null, null);
        this.value = new NamedMatrixIntervalList (value, name);
    }

    public NamedMatrixIntervalList getValue () {
        NamedMatrixIntervalList newValue = new NamedMatrixIntervalList (value);

        //if this variable contains any literal representing variables they are recursiveli expanded
        String varName;
        Variable var;
        for (int i=0;i<newValue.size();++i) {
            if ((varName=newValue.get(i).getName()) != null) {
                var = getVariable(varName);
                if (var != null) {
                    newValue.remove(i);
                    newValue.addAll(i, var.getValue());
                }
            }
        }

        return newValue;
    }

    @Override
    public IntervalList _compute(IntervalList input) throws Exception {
        return new NamedMatrixIntervalList (value);
    }

    @Override
    public String getHelp() {
        if (getHelp().equals("")) return "";

        StringBuffer result = new StringBuffer();
        result.append("\nName: ").append(getSymbol()).append("\nType: variable");
        if (isConstant()) result.append("\nCannot be changed by user");
        else result.append("\nCan be changed by user");
        result.append("\nHelp: ").append(getHelp()).append('\n');
        return result.toString();
    }
}